package creim.analysis.audio;

import java.io.File;
import java.io.IOException;
import creim.analysis.audio.spotifyapi.SpotifyAPI;
import creim.util.Logger;
import creim.util.Util;



public class AudioAnalyzer {
	
	private File    file;
	private String  spotifyTrackID;
	private boolean spotifySupport;
	private File    songfile;
	
	public AudioAnalyzer(String spotifyTrackID) {
		this.spotifyTrackID = spotifyTrackID;
		this.spotifySupport = true;
		this.songfile = null;
	}
	
	public AudioAnalyzer(File file) {
		throw new UnsupportedOperationException("Not impl.");
	}

	public String getID() {
		if (spotifyTrackID != null)
			return spotifyTrackID.substring(spotifyTrackID.lastIndexOf(":")+1, spotifyTrackID.length());
		return null;
	}

	public AudioAnalysisResult analyze(int zonespertime, int zonesperfreq) throws IOException {
		String trackid = spotifyTrackID.substring(spotifyTrackID.lastIndexOf(":")+1);
		
		Logger.info("Initializing Spotify API ... ");
		SpotifyAPI api = new SpotifyAPI();
		File trackdir = api.fetchAllAndStore(trackid);
		
		Logger.info("Analyzing song ... ");
		String filename = trackid + ".mp3";
		songfile = new File(trackdir, filename);
		

		AudioFileAnalyzer an = new AudioFileAnalyzer(songfile);
		double[][] data = an.logSpectrum(true);
		double[][] zones = Util.minmaxScale(data, zonespertime, zonesperfreq);
		double[] std = an.stdFrequencies(data);
		double[] meanf = an.meanFrequencies(data);
		double[] meanat = an.meanAmplitudesPerTime(data);
		double[] meanaf = an.meanAmplitudesPerFrequency(data);
		an.close();
		
		
		AudioAnalysisResult res = new AudioAnalysisResult();
		res.musicdata = data;
		res.zones = zones;
		res.stdFrequencies = std;
		res.meanFrequencies = meanf;
		res.meanAmplitudesPerTime = meanat;
		res.meanAmplitudesPerFrequency = meanaf;
		res.spotifyanalysis = api.getAudioFeatureObject(trackid);
		res.spotifypreview = api.getTrackById(trackid).get("preview_url").toString();
		return res;
	}
	
	
	
	public void play() {
		AudioFileAnalyzer an = new AudioFileAnalyzer(songfile);
		an.play();
	}
}
