package creim.art.artist.ea;

import creim.analysis.audio.spotifyapi.SpotifyAudioFeatures;
import creim.util.Logger;
import eval.ea.EA;
import eval.models.Genotype;
import eval.models.Phenotype;
import eval.models.Population;
import eval.util.Config;
import eval.util.Utils;
import java.awt.Dimension;
import java.util.ArrayList;
import javalib.util.RandomUtil;

/**
 *
 * @author Viktor
 */
public class ArtistEA extends EA {

	private final Dimension imagedim;
	private final SpotifyAudioFeatures[] features;
	private ArrayList<ArtistGenotype> seed;
	
	public ArtistEA(Config eacfg, Dimension imagedim, SpotifyAudioFeatures[] features) {
		super(eacfg);
		this.imagedim = imagedim;
		this.features = features;
		this.seed = new ArrayList<>();
	}
	
	
	public void seedPopulation(ArtistGenotype genotype) {
		this.seed.add(genotype);
	}
	
	
	public void seedPopulation(ArrayList<ArtistGenotype> genotypes) {
		seed.addAll(genotypes);
	}
	

	@Override
	public Population generateInitalPopulation(int size) {
		Population newpopulation = new Population(size);
		boolean log = Logger.log;
		if (!log)
			Logger.log = true;
		if (seed.isEmpty()) {
			Logger.debug("Generating random population. Size: "+size);
			for (int i = 0; i < size; i++) {
				ArtistGenotype genotype = ArtistGenotype.generateRandom();
				ArtistPhenotype phenotype = new ArtistPhenotype(genotype, imagedim, features);
				newpopulation.add(phenotype);
			}
		}
		else {
			while (newpopulation.getCount() < size) {
				int index = Utils.randomInt(seed.size());
				ArtistGenotype genoseed = seed.get(index);
				Logger.debug("Seeding population with: "+genoseed);
				ArtistGenotype genotype = (ArtistGenotype)genoseed.copy();
				ArtistPhenotype phenotype = new ArtistPhenotype(genotype, imagedim, features);
				newpopulation.add(phenotype);
			}
		}
		if (!log)
			Logger.log = false;
		return newpopulation;
	}

	
	@Override
	public Phenotype generatePhenotype(Genotype g) {
		ArtistPhenotype phenotype = new ArtistPhenotype((ArtistGenotype)g, imagedim, features);
		return phenotype;
	}
	
	
}
