package creim.art.artist.ea;


import creim.util.DomainFloat;
import creim.util.Logger;
import eval.models.Crossover;
import eval.models.Genotype;
import eval.util.Utils;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.EnumMap;
import java.util.Map;
import java.util.Objects;
import javalib.util.RandomUtil;



public class ArtistGenotype extends Genotype {

	public static double mutationPressure = 20.0;
	public Map<ToolParameter, GenotypeEntry> map = new EnumMap<>(ToolParameter.class);
	
	
	public ArtistGenotype() {
		
	}
	
	public ArtistGenotype(double mutationPressure) {
		this.mutationPressure = mutationPressure;
	}
	
	public void setMutationPressure(double mutationPressure) {
		this.mutationPressure = mutationPressure;
	}
	
	
	public static ArtistGenotype generateRandom() {
		ArtistGenotype genotype = new ArtistGenotype();
		for (ToolParameter tp : ToolParameter.values()) {
			DomainFloat dom = ToolParameter.getOutputDomain(tp);
//			genotype.map.put(tp, new GenotypeEntry(FeatureVariable.randomNumericVariable(), Utils.randomFloat(0, dom.min), Utils.randomFloat(0, dom.max)));
			genotype.map.put(tp, new GenotypeEntry(FeatureVariable.randomNumericVariable(), Utils.randomFloat(dom.min, dom.max), Utils.randomFloat(dom.min, dom.max)));
		}

		return genotype;
	}
	
	
	public GenotypeEntry getEntry(ToolParameter parameter) {
		return map.get(parameter);
	}
	

	@Override
	public Genotype copy() {
		ArtistGenotype genotype = new ArtistGenotype();
		for (ToolParameter tp : map.keySet()) {
			GenotypeEntry newEntry = new GenotypeEntry(map.get(tp));
			genotype.map.put(tp, newEntry);
		}
		return genotype;
	}

	@Override
	public Genotype crossover(Genotype g, Crossover type) {
		ArtistGenotype genotype = (ArtistGenotype)g;
		switch (type) {
			case ONE_POINT: return onePointCrossover(genotype);
			case UNIFORM: return uniformCrossover(genotype);
			default: throw new AssertionError(type.name());
		}
	}
	
	
	private ArtistGenotype onePointCrossover(ArtistGenotype g) {
		ArtistGenotype genotype = new ArtistGenotype();
		
		int point = Utils.randomInt(Math.min(this.map.size(), g.map.size()));
		
		ToolParameter[] keys = this.map.keySet().toArray(new ToolParameter[0]);
		for (int i = 0; i < point; i++) {
			GenotypeEntry newEntry = new GenotypeEntry(this.map.get(keys[i]));
			genotype.map.put(keys[i], newEntry);
		}
		
		ToolParameter[] gkeys = g.map.keySet().toArray(new ToolParameter[0]);
		for (int i = point; i < gkeys.length; i++) {
			GenotypeEntry newEntry = new GenotypeEntry(g.map.get(gkeys[i]));
			genotype.map.put(gkeys[i], newEntry);
		}
		
		return genotype;
	}
	
	private ArtistGenotype uniformCrossover(ArtistGenotype g) {
		ArtistGenotype genotype = new ArtistGenotype();
		
		int length = Math.max(this.map.size(), g.map.size());
		ToolParameter[] keys = this.map.keySet().toArray(new ToolParameter[0]);
		ToolParameter[] gkeys = g.map.keySet().toArray(new ToolParameter[0]);
		
		for (int i = 0; i < length; i++) {
			boolean b = Utils.randomEvent();
			if (b) {
				if (i >= keys.length)
					continue;
				GenotypeEntry newEntry = new GenotypeEntry(this.map.get(keys[i]));
				genotype.map.put(keys[i], newEntry);
			} else {
				if (i >= gkeys.length)
					continue;
				GenotypeEntry newEntry = new GenotypeEntry(g.map.get(gkeys[i]));
				genotype.map.put(gkeys[i], newEntry);
			}
		}
		
		return genotype;
	}
	

	@Override
	public void mutate(double prob) {
		if (Utils.randomEvent(1.0 - prob))
			return;
		
		int index = Utils.randomInt(map.size());
		ToolParameter key = null;
		GenotypeEntry e = null;
		int i = 0;
		for (ToolParameter tp : map.keySet()) {
			if (i++ == index) {
				key = tp;
				e = map.get(key);
				break;
			}
		}
//		ToolParameter key = RandomUtil.RNG.nextBoolean() ? ToolParameter.ColorDeg : ToolParameter.ColorDist;
//		GenotypeEntry e = map.get(key);
		if (e == null)
			throw new AssertionError("GenotypeEntry is null");
		
		double d = Utils.randomDouble();
		if (d < 0.2) {
			e.variable = FeatureVariable.randomNumericVariable();
			DomainFloat dom = FeatureVariable.getInputDomain(e.variable);
			e.inputmin = dom.min;
			e.inputmax = dom.max;
		} else {
			DomainFloat dom = ToolParameter.getOutputDomain(key);
			double mval = Utils.randomDouble(-mutationPressure, mutationPressure);
			if (Utils.randomEvent()) {
				e.outputmax += mval;
				e.outputmax = Math.min(dom.max, Math.max(dom.min, e.outputmax));
			} else {
				e.outputmin += mval;
				e.outputmin = Math.min(dom.max, Math.max(dom.min, e.outputmin));
			}
		}
	}

	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		sb.append("ArtistGenotype {\n");
		for (ToolParameter tp : map.keySet()) {
			sb.append("\t");
			sb.append(tp);
			sb.append("  =>  ");
			sb.append(map.get(tp));
			sb.append("\n");
		}
		sb.append("}");
		
		return sb.toString();
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 67 * hash + (int) (Double.doubleToLongBits(this.mutationPressure) ^ (Double.doubleToLongBits(this.mutationPressure) >>> 32));
		hash = 67 * hash + Objects.hashCode(this.map);
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final ArtistGenotype other = (ArtistGenotype) obj;
		if (Double.doubleToLongBits(this.mutationPressure) != Double.doubleToLongBits(other.mutationPressure)) {
			return false;
		}
		if (!this.map.keySet().equals(other.map.keySet()))
			return false;
		for (ToolParameter k : map.keySet()) {
			if (!this.map.get(k).equals(other.map.get(k))) {
				return false;
			}
		}
		return true;
	}
	
	
	

	
	public void serialize(File outputfile) throws IOException {
		FileWriter writer = new FileWriter(outputfile);
		for (ToolParameter tp : map.keySet()) {
			GenotypeEntry e = map.get(tp);
			String line = String.format("%s,%s,%f,%f,%f,%f,%f,%f\n", tp.toString(), e.variable, e.inputmin, e.inputmax, e.outputmin, e.outputmax, e.thresholdmin, e.thresholdmax);
			writer.write(line);
		}
		writer.close();
	}
	
	
	public void deserialize(File inputfile) throws IOException {
		BufferedReader reader = new BufferedReader(new FileReader(inputfile));
		String line;
		while ((line = reader.readLine()) != null) {
			line = line.trim();
			if (line.isEmpty())
				continue;
			String[] parts = line.split(",");
			String keyname = parts[0].trim();
			ToolParameter key;
			try {
				key = ToolParameter.valueOf(keyname);
			} catch(IllegalArgumentException e) {
				Logger.debug("Invalid key: "+keyname);
				continue;
			}
			FeatureVariable variable = FeatureVariable.valueOf(parts[1].trim());
			float inputmin = Float.parseFloat(parts[2].trim());
			float inputmax = Float.parseFloat(parts[3].trim());
			float outputmin = Float.parseFloat(parts[4].trim());
			float outputmax = Float.parseFloat(parts[5].trim());
			
			float thresholdmin = -1;
			if (parts.length >= 7)
				thresholdmin = Float.parseFloat(parts[6].trim());
			
			float thresholdmax = -1;
			if (parts.length >= 8)
				thresholdmax = Float.parseFloat(parts[7].trim());
			
			GenotypeEntry value = new GenotypeEntry(variable, inputmin, inputmax, outputmin, outputmax, thresholdmin, thresholdmax);
			map.put(key, value);
		}
		reader.close();
	}
}
