package creim.art.artist.ea;

import creim.util.DomainFloat;
import creim.util.Util;
import java.util.Objects;

/**
 *
 * @author Viktor
 */
public class GenotypeEntry {
	
	public FeatureVariable variable;
	
	public float inputmin;
	public float inputmax;
	public float outputmin;
	public float outputmax;
	public float thresholdmin;
	public float thresholdmax;
	public boolean ignoreoutputmin;
	public boolean ignoreoutputmax;

	public GenotypeEntry(FeatureVariable variable, float outputmin, float outputmax) {
		this(variable, outputmin, outputmax, false, false);
	}
	
	public GenotypeEntry(FeatureVariable variable, float outputmin, float outputmax, boolean ignoreoutputmin, boolean ignoreoutputmax) {
		this(variable, outputmin, outputmax, ignoreoutputmin, ignoreoutputmax, -1, -1);
	}
	
	public GenotypeEntry(FeatureVariable variable, float outputmin, float outputmax, boolean ignoreoutputmin, boolean ignoreoutputmax, float thresholdmin, float thresholdmax) {
		this.variable = variable;
		DomainFloat dom = FeatureVariable.getInputDomain(variable);
		this.inputmin = dom == null ? 0 : dom.min;
		this.inputmax = dom == null ? 0 : dom.max;
		this.outputmin = outputmin;
		this.outputmax = outputmax;
		this.ignoreoutputmin = ignoreoutputmin;
		this.ignoreoutputmax = ignoreoutputmax;
		this.thresholdmin = thresholdmin;
		this.thresholdmax = thresholdmax;
	}
	
	public GenotypeEntry(FeatureVariable variable, float inputmin, float inputmax, float outputmin, float outputmax) {
		this(variable, inputmin, inputmax, outputmin, outputmax, -1, -1);
	}
	
	public GenotypeEntry(FeatureVariable variable, float inputmin, float inputmax, float outputmin, float outputmax, float thresholdmin, float thresholdmax) {
		this.variable = variable;
		this.inputmin = inputmin;
		this.inputmax = inputmax;
		this.outputmin = outputmin;
		this.outputmax = outputmax;
		this.thresholdmin = thresholdmin;
		this.thresholdmax = thresholdmax;
	}
	
	/**
	 * Copy-constructor
	 * @param ge 
	 */
	public GenotypeEntry(GenotypeEntry ge) {
		this.variable = ge.variable;
		this.inputmin = ge.inputmin;
		this.inputmax = ge.inputmax;
		this.outputmin = ge.outputmin;
		this.outputmax = ge.outputmax;
	}

	public void setOutputmin(float outputmin) {
		this.outputmin = outputmin;
	}

	public void setOutputmax(float outputmax) {
		this.outputmax = outputmax;
	}

	@Override
	public String toString() {
		return "GE{" + "var=" + variable + ", in=[" + inputmin + ", " + inputmax + "], out=[" + outputmin + ", " + outputmax + "]}";
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 53 * hash + Objects.hashCode(this.variable);
		hash = 53 * hash + Float.floatToIntBits(this.inputmin);
		hash = 53 * hash + Float.floatToIntBits(this.inputmax);
		hash = 53 * hash + Float.floatToIntBits(this.outputmin);
		hash = 53 * hash + Float.floatToIntBits(this.outputmax);
		return hash;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final GenotypeEntry other = (GenotypeEntry) obj;
		if (!Util.almostEqual(inputmin, other.inputmin, 0.0001f)) {
			return false;
		}
		if (!Util.almostEqual(inputmax, other.inputmax, 0.0001f)) {
			return false;
		}
		if (!Util.almostEqual(outputmin, other.outputmin, 0.0001f)) {
			return false;
		}
		if (!Util.almostEqual(outputmax, other.outputmax, 0.0001f)) {
			return false;
		}
		if (this.variable != other.variable) {
			return false;
		}
		return true;
	}
}