package creim.art.artist.ea;

import creim.analysis.audio.spotifyapi.SpotifyAudioFeatures;
import creim.art.artist.Artist;
import creim.art.painter.OvalPainter;
import creim.art.painter.ClearPainter;
import creim.art.painter.CurveBrushPainter;
import creim.art.painter.Painter;
import creim.art.painter.PolygonPainter;
import creim.art.painter.RectanglePainter;
import creim.art.painter.StraightBrushPainter;
import creim.graphics.colors.ColorPalette;
import creim.graphics.effect.BlurEffect;
import creim.graphics.effect.OilifyEffect;
import creim.util.DomainFloat;
import creim.util.Logger;
import creim.util.Util;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Shape;
import java.awt.geom.CubicCurve2D;
import java.awt.image.BufferedImage;
import java.lang.reflect.Field;
import java.util.ArrayList;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;


/**
 *
 * @author Viktor
 */
public class MappingArtist extends Artist {
	
	private static final Field[] SPOTIFY_FEATURE_FIELDS = SpotifyAudioFeatures.class.getDeclaredFields();

	private SpotifyAudioFeatures features;
	private int margin;
	private boolean doPaint;
	
	public ArtistGenotype genotype;
	public Color[] palette;
	public int colordeg;
	public int colordist;
	public float colorsmin;
	public float colorsmax;
	public float colorS;
	public float colorbmin;
	public float colorbmax;
	public float colorB;
	
	public int sbpminwidth;
	public int sbpmaxwidth;
	public int sbpminlength;
	public int sbpmaxlength;
	public float sbpminrotation;
	public float sbpmaxrotation;
	public int sbpstrokes;
	
	public int cbpminwidth;
	public int cbpmaxwidth;
	public int cbpstrokes;
	
	public int rstrokes;
	public int rminwidth;
	public int rmaxwidth;
	public int rminlength;
	public int rmaxlength;
	
	public int ostrokes;
	public int ominwidth;
	public int omaxwidth;
	public int ominlength;
	public int omaxlength;
	
	public int pstrokes;
	public int pminpoints;
	public int pmaxpoints;
	
	public int strokes;
	public ArrayList<Painter> painters;
	
	
	public MappingArtist(Dimension imagedim, SpotifyAudioFeatures features, ArtistGenotype genotype) {
		this(imagedim, features, genotype, true);
	}
	
	public MappingArtist(Dimension imagedim, SpotifyAudioFeatures features, ArtistGenotype genotype, boolean doPaint) {
		super(imagedim.width, imagedim.height);
		this.features = features;
		this.genotype = genotype;
		this.margin = (int)(Math.max(width, height) * 0.1);
		this.painters = new ArrayList<>();
		this.doPaint = doPaint;
		
		this.setup();
	}

	
	private void setup() {
		setupPalette();
		
		try {
			sbpminwidth  = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.StraightBrushWidthMin)));
			sbpmaxwidth  = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.StraightBrushWidthMax)));
			sbpminlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.StraightBrushLengthMin)));
			sbpmaxlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.StraightBrushLengthMax)));
			sbpminrotation = (float)calculate(genotype.getEntry(ToolParameter.StraightBrushRotationMin));
			sbpmaxrotation = (float)calculate(genotype.getEntry(ToolParameter.StraightBrushRotationMax));
			sbpstrokes = (int)Math.max(0, calculate(genotype.getEntry(ToolParameter.StraightBrushStrokes)));
			Logger.debug("Straight brush strokes: "+sbpstrokes);
			for (int i = 0; i < sbpstrokes; i++) {
				int brushx = RandomUtil.nextInt(-margin, this.width+margin);
				int brushy = RandomUtil.nextInt(-margin, this.height+margin);
				int brushwidth = RandomUtil.nextInt(sbpminwidth, sbpmaxwidth);
				int brushlength = RandomUtil.nextInt(sbpminlength, sbpmaxlength);
				Color brushcolor = palette[RandomUtil.nextInt(palette.length)];
				float brushrotation = RandomUtil.nextFloat(sbpminrotation, sbpmaxrotation);

				StraightBrushPainter sbp = new StraightBrushPainter(brushx, brushy, brushwidth, brushlength, brushcolor, brushrotation);
				painters.add(sbp);
			}
		} catch (Exception e) {}

		try {
			cbpminwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.CurveBrushWidthMin)));
			cbpmaxwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.CurveBrushWidthMax)));
			cbpstrokes = (int)Math.max(0, calculate(genotype.getEntry(ToolParameter.CurveBrushStrokes)));
			Logger.debug("Curve brush strokes: "+cbpstrokes);
			for (int i = 0; i < cbpstrokes; i++) {
				int brushwidth = RandomUtil.nextInt(cbpminwidth, cbpmaxwidth);
				Color brushcolor = palette[RandomUtil.nextInt(palette.length)];
				int x0 = RandomUtil.nextInt(-margin, this.width+margin);
				int y0 = RandomUtil.nextInt(-margin, this.height+margin);
				int cx0 = RandomUtil.nextInt(-margin, this.width+margin);
				int cy0 = RandomUtil.nextInt(-margin, this.height+margin);
				int cx1 = RandomUtil.nextInt(-margin, this.width+margin);
				int cy1 = RandomUtil.nextInt(-margin, this.height+margin);
				int x1 = RandomUtil.nextInt(-margin, this.width+margin);
				int y1 = RandomUtil.nextInt(-margin, this.height+margin);
				Shape shape = new CubicCurve2D.Double(x0, y0, cx0, cy0, cx1, cy1, x1, y1);

				CurveBrushPainter cbp = new CurveBrushPainter(brushwidth, brushcolor);
				cbp.addShape(shape);
				painters.add(cbp);
			}
		} catch (Exception e) {}
		
		try {
			rstrokes = (int)Math.max(0, calculate(genotype.getEntry(ToolParameter.RectangleStrokes)));
			rminwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.RectangleWidthMin)));
			rmaxwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.RectangleWidthMax)));
			rminlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.RectangleLengthMin)));
			rmaxlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.RectangleLengthMax)));
			Logger.debug("Rectangle strokes: "+rstrokes);
			for (int i = 0; i < rstrokes; i++) {
				int x = RandomUtil.nextInt(-margin, this.width+margin);
				int y = RandomUtil.nextInt(-margin, this.height+margin);
				int w = RandomUtil.nextInt(rminwidth, rmaxwidth);
				int h = RandomUtil.nextInt(rminlength, rmaxlength);
				Color c = palette[RandomUtil.nextInt(palette.length)];

				RectanglePainter rp = new RectanglePainter(x, y, w, h, 0);
				rp.color = c;
				painters.add(rp);
			}
		} catch (Exception e) {}

		try {
			ostrokes = (int)Math.max(0, calculate(genotype.getEntry(ToolParameter.OvalStrokes)));
			ominwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.OvalWidthMin)));
			omaxwidth = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.OvalWidthMax)));
			ominlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.OvalLengthMin)));
			omaxlength = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.OvalLengthMax)));
			Logger.debug("Oval strokes: "+ostrokes);
			for (int i = 0; i < ostrokes; i++) {
				int x = RandomUtil.nextInt(-margin, this.width+margin);
				int y = RandomUtil.nextInt(-margin, this.height+margin);
				int w = RandomUtil.nextInt(ominwidth, omaxwidth);
				int h = RandomUtil.nextInt(ominlength, omaxlength);
				Color c = palette[RandomUtil.nextInt(palette.length)];

				OvalPainter op = new OvalPainter(x, y, w, h);
				op.color = c;
				painters.add(op);
			}
		} catch (Exception e) {}
		
		try {
			pstrokes = (int)Math.max(0, calculate(genotype.getEntry(ToolParameter.PolygonStrokes)));
			pminpoints = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.PolygonPointsMin)));
			pmaxpoints = (int)Math.max(1, calculate(genotype.getEntry(ToolParameter.PolygonPointsMax)));
			Logger.debug("Polygon strokes: "+pstrokes);
			for (int i = 0; i < pstrokes; i++) {
				PolygonPainter pp = new PolygonPainter();
				int points = RandomUtil.nextInt(pminpoints, pmaxpoints);
				for (int p = 0; p < points; p++) {
					int x = RandomUtil.nextInt(-margin, this.width+margin);
					int y = RandomUtil.nextInt(-margin, this.height+margin);
					pp.addPolygonPoint(x, y);
				}
				Color c = palette[RandomUtil.nextInt(palette.length)];
				pp.color = c;
				painters.add(pp);
			}
		} catch (Exception e) {}
		
		strokes = painters.size();
		Util.randomize(painters);
	}
	
	public void setupPalette() {
		colordeg  = (int)calculate(genotype.getEntry(ToolParameter.ColorDeg));
		colordist = (int)calculate(genotype.getEntry(ToolParameter.ColorDist));
		colorsmin = (float)calculate(genotype.getEntry(ToolParameter.ColorSMin));		
		colorsmax = (float)calculate(genotype.getEntry(ToolParameter.ColorSMax));
		colorbmin = (float)calculate(genotype.getEntry(ToolParameter.ColorBMin));		
		colorbmax = (float)calculate(genotype.getEntry(ToolParameter.ColorBMax));
		palette = ColorPalette.generateFullyRandomAdjacentPalette(colordeg, colordist, colorsmin, colorsmax, colorbmin, colorbmax);
//		palette = ColorPalette.generateFullyRandomAdjacentPalette(colordeg, colordist, colorS, colorB);
//		palette = ColorPalette.generateFullyRandomAdjacentPalette(colordeg, colordist, 1, 1, 1, 1);
//		palette = RYBColorWheel.adjacent(colordeg, colordist);
	}
	
	
	@Override
	public BufferedImage paint() {
		Logger.debug(features.toString());
		Logger.debug(ColorPalette.toString(palette));
		
		ClearPainter cp = new ClearPainter(width, height, Color.white);
		cp.paintOn(image);
		
		for (Painter p : painters) {
			if (p instanceof StraightBrushPainter)
				paintStraightBrushStroke((StraightBrushPainter)p);
			else if (p instanceof CurveBrushPainter)
				paintCruveBrushStroke((CurveBrushPainter)p);
			else if (p instanceof RectanglePainter)
				paintSquare((RectanglePainter)p);
			else if (p instanceof OvalPainter)
				paintOval((OvalPainter)p);
			else if (p instanceof PolygonPainter)
				paintPolygon((PolygonPainter)p);
			else
				throw new AssertionError("Missing painter");
		}
		
		int oilradius = 5;
		int oilintensity = 15;
		Logger.debug("Oil ["+oilradius+", "+oilintensity+"]");
		OilifyEffect oil = new OilifyEffect(image, oilradius, oilintensity);
		image = oil.apply();
		
		int blurradius = (int) calculate(genotype.getEntry(ToolParameter.BlurEffect));
		Logger.debug("Blur ["+blurradius+"]");
		if (blurradius > 0) {
			BlurEffect blur = new BlurEffect(image, blurradius);
			image = blur.apply();
		}
		
		return image;
	}
	
	
	public double calculate(GenotypeEntry entry) {
		return calculateStatic(entry, features);
	}
	
	public static double calculateStatic(GenotypeEntry entry, SpotifyAudioFeatures features) {
		try {
			FeatureVariable featurevar = entry.variable;
			if (featurevar == FeatureVariable.RANDOM_GENERATOR) {
				double input = RandomUtil.nextDouble();
				DomainFloat vardom = FeatureVariable.getInputDomain(featurevar);
				double out = MathUtil.scale(input, vardom.min, vardom.max, entry.outputmin, entry.outputmax);
				return out;
			}
			
			for (Field f : SPOTIFY_FEATURE_FIELDS) {
				if (!f.getName().equalsIgnoreCase(featurevar.name()))
					continue;
				
				Object value = features.getClass().getField(f.getName().toLowerCase()).get(features);
				if (value instanceof Number) {
					double input = ((Number) value).doubleValue();
					DomainFloat vardom = FeatureVariable.getInputDomain(featurevar);
					
					if (entry.thresholdmin >= 0 && input < entry.thresholdmin)
						return entry.outputmin;
					if (entry.thresholdmax >= 0 && input >= entry.thresholdmax)
						return entry.outputmax;
					
					return MathUtil.scale(input, vardom.min, vardom.max, entry.outputmin, entry.outputmax);
				}
			}
		} catch(Exception e) {
		}
		throw new UnsupportedOperationException("Oops... Could not find value: "+entry);
	}
	
	
	public void paintStraightBrushStroke(StraightBrushPainter painter) {
		Logger.debug(String.format("StraightBrushPainter [%4d, %4d, %4d, %4d, %s, %.2f]", painter.x, painter.y, painter.brushwidth, painter.length, ColorPalette.toString(painter.brushcolor), painter.rotationdegrees));
		painter.paintOn(image);
	}
	
	public void paintCruveBrushStroke(CurveBrushPainter painter) {
		CubicCurve2D.Double s = (CubicCurve2D.Double) painter.shapes.get(0);
		Logger.debug(String.format("CurveBrushPainter [%4d, %s, %4d, %4d, %4d, %4d, %4d, %4d, %4d, %4d]", (int)painter.brushwidth, ColorPalette.toString(painter.brushcolor), (int)s.x1, (int)s.y1, (int)s.ctrlx1, (int)s.ctrly1, (int)s.ctrlx2, (int)s.ctrly2, (int)s.x2, (int)s.y2));
		painter.paintOn(image);
	}
	
	public void paintOval(OvalPainter painter) {
		Logger.debug(String.format("OvalPainter [%4d, %4d, %4d, %4d, %s, %.2f, %s]", painter.x, painter.y, painter.w, painter.h, ColorPalette.toString(painter.color), painter.outlineThickness, ColorPalette.toString(painter.outlineColor)));
		painter.paintOn(image);
	}
	
	public void paintSquare(RectanglePainter painter) {
		Logger.debug(String.format("RectanglePainter [%4d, %4d, %4d, %4d, %s, %.2f, %s]", painter.x, painter.y, painter.w, painter.h, ColorPalette.toString(painter.color), painter.outlineThickness, ColorPalette.toString(painter.outlineColor)));
		painter.paintOn(image);
	}
	
	public void paintPolygon(PolygonPainter painter) {
		Logger.debug(String.format("PolygonPainter [%s, %s, %.2f, %s]", painter.points, ColorPalette.toString(painter.color), painter.outlineThickness, ColorPalette.toString(painter.outlineColor)));
		painter.paintOn(image);
	}
	
}
