package creim.art.models;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.geom.CubicCurve2D;
import javalib.util.RandomUtil;
import creim.oldea.EAMutable;
import creim.graphics.colors.ColorPalette;
import java.io.Serializable;



public abstract class PainterModel implements EAMutable, Serializable, ModelCoding {
	
	/** Random values range from (-margin, imagedim + margin)*/
	public int margin;
	/** Dimension of output image */
	public Dimension imagedim;
	/** Color palette to be used */
	public Color[] palette;
	/** Current index in palette selects color */
	public int paletteIndex;
	/** Current index in palette selects outline color */
	public int paletteOutlineIndex;

	/**
	 * Set margin for random values.
	 * Random values range from (-margin, imagedim + margin)
	 * @param margin 
	 */
	public void setMargin(int margin) {
		this.margin = margin;
	}
	
	/**
	 * Set image output dimension.
	 * @param imagedim 
	 */
	public void setImageDim(Dimension imagedim) {
		this.imagedim = imagedim;
	}
	
	/**
	 * Set color palette to be used. Usually random indices from this palette are used.
	 * @param palette 
	 */
	public void setPalette(Color[] palette) {
		this.palette = palette;
	}
	
	/**
	 * Randomize model values to draw random images.
	 */
	public abstract void randomize();
	
	public static PainterModel randomModel(Dimension imagedim, int margin, Color[] palette) {
		int totalpainters = 6;
		
		int selection = RandomUtil.nextInt(RandomUtil.randBool(0.8) ? 2 : totalpainters);
		switch (selection) {
			case 0:
			{
				StraightBrushModel model = new StraightBrushModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				model.randomize();

				return model;
			}
			case 1:
			{
				CurveBrushModel model = new CurveBrushModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				model.randomize();
				return model;
			}
			case 2:
			{
				SquareModel model = new SquareModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				
				model.randomize();
				return model;
			}
			case 3:
			{
				CircleModel model = new CircleModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				model.randomize();
				return model;
			}
			case 4:
			{
				CircleBrushModel model = new CircleBrushModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				model.randomize();
				return model;
			}
			case 5:
			{
				PolygonModel model = new PolygonModel();
				model.setImageDim(imagedim);
				model.setPalette(palette);
				model.setMargin(margin);
				model.randomize();
				return model;
			}
			default:
				throw new RuntimeException("Something went wronk: "+selection);
		}
	}
	
	
}
