package creim.art.painter;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.FlatteningPathIterator;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.graphics.colors.ColorUtil;
import java.awt.Dimension;



public class CurveBrushPainter extends Painter {
	
	public float brushwidth;
	public Color brushcolor;
	public ArrayList<Shape> shapes;
	private int brushhairs;
	private int[][] brushpoints;
	private boolean brushcreated = false;

	public CurveBrushPainter() {
		this(25, Color.BLACK);
	}

	public CurveBrushPainter(float brushwidth, Color brushcolor) {
		this(brushwidth, brushcolor, null);
	}
	
	public CurveBrushPainter(float brushwidth, Color brushcolor, ArrayList<Shape> shapes) {
		this.brushwidth = brushwidth;
		this.brushcolor = brushcolor;
		if (shapes == null)
			this.shapes = new ArrayList<>();
		else
			this.shapes = shapes;
		
	}
	
	private void createBrush() {
		this.brushhairs = (int)(this.brushwidth * Math.log(this.brushwidth) * 30);
		this.brushpoints = new int[brushhairs][2];

		for (int i = 0; i < brushhairs; i++) {
			double t = 2 * Math.PI * RandomUtil.nextDouble();
			double u = RandomUtil.nextDouble() + RandomUtil.nextDouble();
			double r = u > 1 ? 2-u : u;
			int px = (int)((Math.cos(t) * r) * this.brushwidth);
			int py = (int)((Math.sin(t) * r) * this.brushwidth);

			int[] p = {px, py};
			brushpoints[i] = p;
		}
	}
	
	public void addShape(Shape s) {
		shapes.add(s);
	}

	@Override
	public void randomize() {
		brushwidth = RandomUtil.nextInt(75);
		brushcolor = palette[RandomUtil.nextInt(palette.length)];
		
		if (!brushcreated)
			createBrush();
		
		int  x1 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		int  y1 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		int cx1 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		int cy1 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		int cx2 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		int cy2 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		int  x2 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		int  y2 = RandomUtil.nextInt(-margin, imagedim.height + margin);

		CubicCurve2D.Double shape = new CubicCurve2D.Double(x1, y1, cx1, cy1, cx2, cy2, x2, y2);
		addShape(shape);
	}

	@Override
	public void mutate(Dimension imagedim, Color[] palette, int margin) {
		
		int p = RandomUtil.nextInt(10);
		
		CubicCurve2D.Double shape = (CubicCurve2D.Double)shapes.get(0);
		if (shape == null) {
			throw new RuntimeException("Shapes is null...");
		}
		
		if (p == 0)
			brushwidth = RandomUtil.nextInt(75);
		else if (p == 1)
			brushcolor = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 2)
			shape.x1 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 3)
			shape.y1 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 4)
			shape.ctrlx1 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 5)
			shape.ctrly1 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 6)
			shape.ctrlx2 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 7)
			shape.ctrly2 = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 8)
			shape.x2 = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 9)
			shape.y2 = RandomUtil.nextInt(-margin, imagedim.height + margin);
	}
	

	@Override
	public BufferedImage paint() {
		throw new RuntimeException("Broken.");
	}

	@Override
	public void paintOn(BufferedImage image) {
		if (!brushcreated)
			createBrush();
		
		Graphics2D g2 = (Graphics2D) image.getGraphics();
		g2.setColor(this.brushcolor);
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		double[] brushstrength = new double[brushhairs];
		double[] brusheffect = new double[brushhairs];
		for (int i = 0; i < brushhairs; i++) {
			int[] p = brushpoints[i];
			double dist = Math.sqrt(p[0] * p[0] + p[1] * p[1]);
			brushstrength[i] = dist > 0.01 ? 1.0 / dist : 0.01;
			brusheffect[i] = RandomUtil.RNG.nextDouble() * 0.1;// * dist;
		}
		

		for (Shape shape : this.shapes) {
			FlatteningPathIterator iter = new FlatteningPathIterator(shape.getPathIterator(new AffineTransform()), 0.5);
			ArrayList<Point> points = new ArrayList<>();
			float[] coords = new float[6];
			while (!iter.isDone()) {
				iter.currentSegment(coords);
				int x = (int) coords[0];
				int y = (int) coords[1];
				points.add(new Point(x, y));
				iter.next();
			}

			for (int s = 0; s < points.size()-1; s++) {
				Point p0 = points.get(s);
				Point p1 = points.get(s+1);

				for (int h = 0; h < brushhairs; h++) {
					double ef = ((Math.sin(brusheffect[h] * s) + 1.0) / 2.0) * brushstrength[h];
					int rgb = (this.brushcolor.getRGB() & 0x00ffffff);
					rgb = ColorUtil.scale(rgb, 1.0 + (ef + RandomUtil.nextGaussian(0.0, 0.05)));
					Color c = new Color(( (rgb & 0x00ffffff) | ((int)(MathUtil.scale(Math.min(ef, 1.0), 0, 1) * 255) << 24)), true);
//					Color c = brushcolor;
					g2.setColor(c);

					int[] p = brushpoints[h];
					g2.drawLine((int)(p[0] + p0.x), (int)(p[1] + p0.y), (int)(p[0] + p1.x), (int)(p[1] + p1.y));
				}

				for (int i = 0; i < brushhairs; i++) {
//					brushstrength[i] *= 0.999;
					brushstrength[i] *= 0.97;
				}
			}
		}


		g2.dispose();
	}


	@Override
	public double[] centerPoint() {
		CubicCurve2D.Double s = (CubicCurve2D.Double)shapes.get(0);
		
		double hmax = Math.max(s.y1, s.y2);
		hmax = Math.max(hmax, s.ctrly1);
		hmax = Math.max(hmax, s.ctrly2);
		
		double hmin = Math.min(s.y1, s.y2);
		hmin = Math.min(hmin, s.ctrly1);
		hmin = Math.min(hmin, s.ctrly2);
		
		double wmax = Math.max(s.x1, s.x2);
		wmax = Math.max(wmax, s.ctrlx1);
		wmax = Math.max(wmax, s.ctrlx2);
		
		double wmin = Math.min(s.x1, s.x2);
		wmin = Math.min(wmin, s.ctrlx1);
		wmin = Math.min(wmin, s.ctrlx2);
		
		return new double[]{(wmax + wmin)/2.0, (hmax + hmin)/2.0};
	}

	@Override
	public double area() {
		CubicCurve2D.Double s = (CubicCurve2D.Double)shapes.get(0);
		
		double hmax = Math.max(s.y1, s.y2);
		hmax = Math.max(hmax, s.ctrly1);
		hmax = Math.max(hmax, s.ctrly2);
		
		double hmin = Math.min(s.y1, s.y2);
		hmin = Math.min(hmin, s.ctrly1);
		hmin = Math.min(hmin, s.ctrly2);
		
		double wmax = Math.max(s.x1, s.x2);
		wmax = Math.max(wmax, s.ctrlx1);
		wmax = Math.max(wmax, s.ctrlx2);
		
		double wmin = Math.min(s.x1, s.x2);
		wmin = Math.min(wmin, s.ctrlx1);
		wmin = Math.min(wmin, s.ctrlx2);
		
		return (wmax - wmin) * (hmax - hmin);
	}
}
