package creim.art.painter;

import creim.oldea.EAMutable;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.image.BufferedImage;
import javalib.util.RandomUtil;



public abstract class Painter implements EAMutable {
	
	public Painter() {}
	
	/** Random values range from (-margin, imagedim + margin)*/
	public int margin;
	/** Dimension of output image */
	public Dimension imagedim;
	/** Color palette to be used */
	public Color[] palette;
	/** Current index in palette selects color */
	public int paletteIndex;
	/** Current index in palette selects outline color */
	public int paletteOutlineIndex;

	/**
	 * Set margin for random values.
	 * Random values range from (-margin, imagedim + margin)
	 * @param margin 
	 */
	public void setMargin(int margin) {
		this.margin = margin;
	}
	
	public abstract double[] centerPoint();
	public abstract double area();
	
	/**
	 * Set image output dimension.
	 * @param imagedim 
	 */
	public void setImageDim(Dimension imagedim) {
		this.imagedim = imagedim;
	}
	
	/**
	 * Set color palette to be used. Usually random indices from this palette are used.
	 * @param palette 
	 */
	public void setPalette(Color[] palette) {
		this.palette = palette;
	}
	
	/**
	 * Randomize model values to draw random images.
	 */
	public abstract void randomize();
	
	public static Painter randomPainter(Dimension imagedim, int margin, Color[] palette) {
		int totalpainters = 6;
		
//		int selection = RandomUtil.nextInt(RandomUtil.randBool(0.8) ? 2 : totalpainters);
		int selection = RandomUtil.nextInt(totalpainters);
		switch (selection) {
			case 0:
			{
				StraightBrushPainter painter = new StraightBrushPainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				painter.randomize();

				return painter;
			}
			case 1:
			{
				CurveBrushPainter painter = new CurveBrushPainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				painter.randomize();
				return painter;
			}
			case 2:
			{
				RectanglePainter painter = new RectanglePainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				
				painter.randomize();
				return painter;
			}
			case 3:
			{
				OvalPainter painter = new OvalPainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				painter.randomize();
				return painter;
			}
			case 4:
			{
				CircleBrushPainter painter = new CircleBrushPainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				painter.randomize();
				return painter;
			}
			case 5:
			{
				PolygonPainter painter = new PolygonPainter();
				painter.setImageDim(imagedim);
				painter.setPalette(palette);
				painter.setMargin(margin);
				painter.randomize();
				return painter;
			}
			default:
				throw new RuntimeException("Something went wronk: "+selection);
		}
	}
	
	
	/**
	 * Paints on new buffered image which is just big enough for the shape.
	 * Empty parts should be transparent
	 * @return 
	 */
	public abstract BufferedImage paint();
	
	
	/**
	 * Paints directly on the input image
	 * @param img 
	 */
	public abstract void paintOn(BufferedImage image); 
	
	
}
