package creim.art.painter;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import javalib.util.RandomUtil;



public class PolygonPainter extends Painter {
	
	public Color color = Color.WHITE;
	public Color outlineColor = Color.BLACK;
	public float outlineThickness = 0.0f;
	public ArrayList<Point> points;

	public PolygonPainter() {
		this.points = new ArrayList<>();
	}

	public PolygonPainter(ArrayList<Point> points) {
		this.points = points;
	}
	
	
	public void addPolygonPoint(int x, int y) {
		points.add(new Point(x, y));
	}
	
	public void addPolygonPoint(Point point) {
		points.add(point);
	}
	
	public Point randomPoint() {
		Point p = new Point();
		p.x = RandomUtil.nextInt(-margin, imagedim.width + margin);
		p.y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		return p;
	}

	@Override
	public void randomize() {
		color = palette[RandomUtil.nextInt(palette.length)];
		outlineColor = palette[RandomUtil.nextInt(palette.length)];
		outlineThickness = RandomUtil.nextInt(5);

		int count = RandomUtil.nextInt(3, 7);
		for (int c = 0; c < count; c++) {
			addPolygonPoint(randomPoint());
		}
	}
	
	
	
	

	@Override
	public void mutate(Dimension imagedim, Color[] palette, int margin) {
		int p = RandomUtil.nextInt(3 + this.points.size());
		if (p == 0)
			color = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 1)
			outlineColor = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 2)
			outlineThickness = RandomUtil.nextInt(5);
		else {
			int i = p - 3;
			Point point = points.get(i);
			if (RandomUtil.RNG.nextBoolean())
				point.x = RandomUtil.nextInt(-margin, imagedim.width + margin);
			else
				point.y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		}
	}
	
	@Override
	public BufferedImage paint() {
		int width = 0;
		int height = 0;
		for (Point p : points) {
			width = Math.max(p.x, width);
			height = Math.max(p.y, height);
		}
		BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		paintOn(image);
		return image;
	}
	
	
	@Override
	public void paintOn(BufferedImage image) {
		Graphics2D g = (Graphics2D)image.getGraphics();
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		paintWithPoints(g);

		g.dispose();
	}
	
	
	private void paintWithPoints(Graphics2D g) {
		
		Polygon polygon = new Polygon();
		for (Point p : points) {	
			polygon.addPoint(p.x, p.y);
		}

		g.setColor(color);
		g.fillPolygon(polygon);

		if (outlineThickness >= 1.0f) {
			g.setColor(outlineColor);
			g.setStroke(new BasicStroke(outlineThickness));
			g.drawPolygon(polygon);
		}
	}
	

	@Override
	public double[] centerPoint() {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public double area() {
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}
}
