package creim.art.painter;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import javalib.util.RandomUtil;
import java.awt.Dimension;



public class RectanglePainter extends Painter {

	public int x;
	public int y;
	public int w;
	public int h;
	public Color color = Color.WHITE;
	public Color outlineColor = Color.BLACK;
	public float outlineThickness;

	public RectanglePainter() {
	}

	public RectanglePainter(int x, int y, int w, int h, float outlineThickness) {
		this.x = x;
		this.y = y;
		this.w = w;
		this.h = h;
		this.outlineThickness = outlineThickness;
	}
	
	
	@Override
	public void randomize() {
		x = RandomUtil.nextInt(-margin, imagedim.width + margin);
		y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		w = RandomUtil.nextInt(-margin, imagedim.width + margin);
		h = RandomUtil.nextInt(-margin, imagedim.height + margin);
		color = palette[RandomUtil.nextInt(palette.length)];
		outlineColor = palette[RandomUtil.nextInt(palette.length)];
		outlineThickness = RandomUtil.nextInt(5);
	}
	
	

	@Override
	public void mutate(Dimension imagedim, Color[] palette, int margin) {
		int p = RandomUtil.nextInt(7);
		if (p == 0)
			x = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 1)
			y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 2)
			w = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 3)
			h = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 4)
			color = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 5)
			outlineColor = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 6)
			outlineThickness = RandomUtil.nextInt(5);
	}


	@Override
	public BufferedImage paint() {
		BufferedImage image = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
		paintOn(image);
		return image;
	}
	

	@Override
	public void paintOn(BufferedImage image) {
		Graphics2D g = (Graphics2D)image.getGraphics();
		

		g.setColor(color);
		g.fillRect(x, y, w, h);

		if (outlineThickness >= 1.0f) {
			g.setColor(outlineColor);
			g.setStroke(new BasicStroke(outlineThickness));
			g.drawRect(x, y, w, h);
		}
		
		g.dispose();
	}


	@Override
	public double[] centerPoint() {
		return new double[]{x + w/2.0, y + h/2.0};
	}

	@Override
	public double area() {
		return w * h;
	}
}
