package creim.art.painter;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import javalib.util.RandomUtil;
import creim.graphics.colors.ColorUtil;
import java.awt.Dimension;



public class StraightBrushPainter extends Painter {
	
	public static final int NORTH = 0;
	public static final int SOUTH = 1;
	public static final int EAST  = 2;
	public static final int WEST  = 3;

	public int x;
	public int y;
	public int brushwidth;
	public int length;
	public Color brushcolor;
	public double rotationdegrees;

	public StraightBrushPainter() {
	}

	public StraightBrushPainter(int x, int y, int brushwidth, int length, Color brushcolor, double rotationdegrees) {
		this.x = x;
		this.y = y;
		this.brushwidth = brushwidth;
		this.length = length;
		this.brushcolor = brushcolor;
		this.rotationdegrees = rotationdegrees;
	}

	@Override
	public void randomize() {
		paletteIndex = RandomUtil.nextInt(palette.length);
		
		x = RandomUtil.nextInt(-margin, imagedim.width + margin);
		y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		brushwidth = RandomUtil.nextInt(100);
		length = RandomUtil.nextInt(imagedim.height + margin);
		brushcolor = palette[paletteIndex];
		rotationdegrees = RandomUtil.nextInt(360);
	}
	
	@Override
	public void mutate(Dimension imagedim, Color[] palette, int margin) {
		int p = RandomUtil.nextInt(6);
		if (p == 0)
			x = RandomUtil.nextInt(-margin, imagedim.width + margin);
		else if (p == 1)
			y = RandomUtil.nextInt(-margin, imagedim.height + margin);
		else if (p == 2)
			brushwidth = RandomUtil.nextInt(100);
		else if (p == 3)
			length = RandomUtil.nextInt(imagedim.height + margin);
		else if (p == 4)
			brushcolor = palette[RandomUtil.nextInt(palette.length)];
		else if (p == 5)
			rotationdegrees = RandomUtil.nextInt(360);
	}
	
	
	
	@Override
	public BufferedImage paint() {
		BufferedImage image = new BufferedImage(brushwidth, length, BufferedImage.TYPE_INT_ARGB);
		paintOn(image);
		return image;
	}
	
	
	@Override
	public void paintOn(BufferedImage image) {
		Graphics2D g2 = (Graphics2D)image.getGraphics();
		
		
		double[] sinf = new double[brushwidth];
		double[] fade = new double[brushwidth];
		
		for (int i = 0; i < brushwidth; i++) {
			sinf[i] = RandomUtil.RNG.nextDouble() * 0.02 * (RandomUtil.RNG.nextGaussian() + 0.5);
			fade[i] = Math.max(0.01, MathUtil.gaussian(i, 1, brushwidth/2.0, brushwidth/3.0));
		}
		AffineTransform oldtrans = g2.getTransform();
		g2.rotate(Math.toRadians(rotationdegrees), x, y);
		
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
	
		for (int t = 0; t < length; t++) {
			for (int i = 0; i < brushwidth; i++) {
				double p = (Math.sin(sinf[i] * t) + 1.0) * fade[i];
				
				int rgb = (brushcolor.getRGB() & 0x00ffffff);
				rgb = ColorUtil.scale(rgb, 1.0 + (p * 0.1 + RandomUtil.nextGaussian(0.0, 0.0005)));
				
				Color c = new Color(( (rgb & 0x00ffffff) | ((int)(MathUtil.scale(Math.min(p, 1.0), 0, 1) * 255) << 24)), true);
				
				g2.setColor(c);
				g2.fillRect(x + i, y + t, 1, 1);
//				g2.drawLine(x, y, x+1, y+1);
			}
			
			for (int i = 0; i < brushwidth; i++)
				fade[i] *= t < length * 0.85 ? 0.997 : 0.9;
		}
		g2.setTransform(oldtrans);
		g2.dispose();
	}

	@Override
	public double[] centerPoint() {
		return new double[]{x + brushwidth/2.0, y + length/2.0};
	}

	@Override
	public double area() {
		return brushwidth * length;
	}
}
