package creim.graphics.colors;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.image.BufferedImage;
import javalib.util.MathUtil;
import creim.gui.ImageFrame;



public class RYBColorWheel {
	
	public static Color[] wheel;
	
	static {
		RYB[] ryb = new RYB[12 * (int)Math.pow(2, 6)];
		
		RYB r = new RYB(1, 0, 0);
		RYB y = new RYB(0, 1, 0);
		RYB b = new RYB(0, 0, 1);
		
		ryb[0 * ryb.length/3] = r;
		ryb[1 * ryb.length/3] = y;
		ryb[2 * ryb.length/3] = b;
		
		for (int diff = ryb.length / 3; diff > 0; diff /= 2) {
			for (int s = 0; s < ryb.length / diff; s++) {
				int i = s * diff;
				if (ryb[i] != null)
					continue;
				int m1 = (i - diff) % ryb.length;
				int m2 = (i + diff) % ryb.length;
				ryb[i] = ColorUtil.mix(ryb[m1], ryb[m2]);
			}
		}
		
		wheel = new Color[ryb.length];
		for (int i = 0; i < ryb.length; i++) {
			wheel[i] = ryb[i].toRGBInterpolated().toColor();
		}
	}
	
	
	public static Color at(double deg) {
		while (deg >= 360) deg -= 360;
		while (deg < 0) deg += 360;
		int index = (int)Math.round((deg / 360.0) * wheel.length);
		return wheel[index];
	}
	
	
	public static Color complementaryColor(double deg) {
		double comp = (deg + 180) % 360;
		int index = (int)Math.round((comp / 360.0) * wheel.length);
		return wheel[index];
	}
	
	
	public static Color[] complementary(double deg) {
		Color[] res = new Color[2];
		res[0] = at(deg);
		res[1] = complementaryColor(deg);
		return res;
	}
	
	
	public static Color[] triad(double deg, double dist) {
		Color[] res = new Color[3];
		res[0] = complementaryColor(deg+dist);
		res[1] = at(deg);
		res[2] = complementaryColor(deg-dist);
		return res;
	}
	
	
	public static Color[] tetrad(double deg, double dist) {
		Color[] res = new Color[4];
		res[0] = at(deg);
		res[1] = at(deg+dist);
		res[2] = complementaryColor(deg);
		res[3] = complementaryColor(deg+dist);
		return res;
	}
	
	
	public static Color[] adjacent(double deg, double dist) {
		Color[] res = new Color[5];
		res[0] = at(deg+dist);
		res[1] = at(deg+dist/2);
		res[2] = at(deg);
		res[3] = at(deg-dist/2);
		res[4] = at(deg-dist);
		return res;
	}
	
	
	public static Color[] scale(Color[] palette, double factor) {
		Color[] res = new Color[palette.length];
		
		for (int i = 0; i < palette.length; i++) {
			int a = (int)Math.max(0, Math.min(255, palette[i].getAlpha() * factor));
			int r = (int)Math.max(0, Math.min(255, palette[i].getRed() * factor));
			int g = (int)Math.max(0, Math.min(255, palette[i].getGreen() * factor));
			int b = (int)Math.max(0, Math.min(255, palette[i].getBlue() * factor));
			
			res[i] = new Color(r, g, b, a);
		}
		
		return res;
	}
	
	
	public static void showColorWheel() {	
		int width = 800;
		int height = 800;
		BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		Graphics2D g = img.createGraphics();
		
		double cx = width / 2.0;
		double cy = height / 2.0;
		double r  = 300;
		
		for (int i = 0; i < wheel.length; i++) {
			double s = (2.0 * Math.PI) * (i / (double)wheel.length);
			double x = Math.cos(s) * r + cx;
			double y = Math.sin(s) * r + cy;
			double ns = (2.0 * Math.PI) * ((i+1) / (double)wheel.length);
			double nx = Math.cos(ns) * r + cx;
			double ny = Math.sin(ns) * r + cy;
			
			Polygon p = new Polygon();
			p.addPoint((int)Math.round(x), (int)Math.round(y));
			p.addPoint((int)Math.round(nx), (int)Math.round(ny));
			p.addPoint((int)cx, (int)cy);
			
			if (wheel[i] == null)
				g.setColor(Color.white);
			else
				g.setColor(wheel[i]);
			g.fillPolygon(p);
		}

		g.dispose();
		ImageFrame frame = new ImageFrame(width, height, img);
		frame.setVisible(true);
	}
	
	
	/**
	 * Output degree for use on RYBColorWheel, representing the heat color.
	 * @param value from 0 to 1, where 1 is hot and 0 is cold
	 * @return 
	 */
	public static double heat(double value) {
		return 240 - MathUtil.scale(value, 0, 1, 0, 240);
	}
	
}
