package creim.util;

import java.util.ArrayList;
import javalib.util.RandomUtil;
import org.apache.commons.math3.stat.StatUtils;
import org.json.simple.JSONObject;



public class Util {
	public static String json(JSONObject json) {
		return json(json.toJSONString());
	}
	
	public static String json(String json) {
		int tabs = 0;
		boolean expectspace = false;
		boolean isinstr = false;
		
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < json.length(); i++) {
			char c = json.charAt(i);
			
			if (expectspace) {
				if (c != ' ') {
					if (!isinstr)
						sb.append(' ');
				}
				expectspace = false;
			}
			if (c == ':') {
				expectspace = true;
			}
			
			if (Character.isLetterOrDigit(c))
				sb.append(c);
			else {
				switch (c) {
					case '"':
						isinstr = !isinstr;
						break;
					case '\\':
						continue;
					case ']':
					case '}':
						tabs--; 
						jsonnewline(sb, tabs);
						break;

					case '{':
					case '[':
						sb.append(c);
						tabs++; 
						jsonnewline(sb, tabs);
						continue;
						
					case ',':
						sb.append(c);
						jsonnewline(sb, tabs);
						continue;
					case '\n':
						jsonnewline(sb, tabs);
						continue;
				}
				sb.append(c);
			}
		}
		return sb.toString();
	}
	
	private static void jsonnewline(StringBuilder sb, int tabs) {
		sb.append('\n');
		for (int j = 0; j < tabs; j++) {
			sb.append('\t');
		}
	}
	
	
	public static double[][] nearestNeighborScale(double[][] in, int outrows, int outcols) {
		double[][] out = new double[outrows][outcols];
		
		double rr = in.length / (double)outrows;
		double cr = in[0].length / (double)outcols;
		
		int px, py;
		for (int r = 0; r < outrows; r++) {
			for (int c = 0; c < outcols; c++) {
				px = (int)Math.floor(r * rr);
				py = (int)Math.floor(c * cr);
				out[r][c] = in[px][py];
			}
		}
		
		return out;
	}
	
	
	public static double[][] minmaxScale(double[][] in, int outrows, int outcols) {
		double[][] out = new double[outrows][outcols];
		
		int rr = in.length / outrows;
		int cr = in[0].length / outcols;
		int rrm = in.length % outrows;
		int crm = in[0].length % outcols;
		
		for (int r = 0; r < outrows; r++) {
			for (int c = 0; c < outcols; c++) {
				int rlen = r < outrows-1 ? rr : rr + rrm;
				int clen = c < outcols-1 ? cr : cr + crm;
				double[] values = new double[rlen * clen];
				
				int i = 0;
				for (int ir = r * rr; ir < r * rr + rlen; ir++) {
					for (int ic = c * cr; ic < c * cr + clen; ic++) {
						values[i++] = in[ir][ic];
					}
				}
				
				double amean = StatUtils.mean(values);
				double gmean = StatUtils.geometricMean(values);
	
				if (r == 0)
					out[r][c] = minAboveZero(values);
				else if (gmean < amean)
					out[r][c] = minAboveZero(values);
				else
					out[r][c] = StatUtils.max(values);
			}
		}
		
		return out;
	}
	
	
	
	public static double minAboveZero(double[] values) {
		double min = StatUtils.max(values);
		for (int i = 0; i < values.length; i++) {
			if (values[i] < min && values[i] != 0)
				min = values[i];
		}
		return min;
	}
	
	
	/**
	 * Fisher-Yates shuffle
	 *
	 * @param <T> generic type
	 * @param array 
	 */
	public static <T> void randomize(T[] array) {
		for (int i = array.length-1; i > 0; i--) {
			int j = RandomUtil.nextInt(i + 1);
			T tmp = array[i];
			array[i] = array[j];
			array[j] = tmp;
		}
	}
	
	/**
	 * Fisher-Yates shuffle
	 *
	 * @param <T> generic type
	 * @param array 
	 */
	public static <T> void randomize(ArrayList<T> array) {
		for (int i = array.size()-1; i > 0; i--) {
			int j = RandomUtil.nextInt(i + 1);
			T tmp = array.get(i);
			array.set(i, array.get(j));
			array.set(j, tmp);
		}
	}
	
	
	
	
	public static boolean almostEqual(double a, double b, double percent) {
		return Math.abs((a - b)/a) < percent;
	}
	
	
	public static boolean almostEqual(float a, float b, float percent) {
		if (Math.abs(a - b) < 1e-7)
			return true;
		return Math.abs((a - b)/a) < percent;
	}

}
